USE AdventureWorks2017;
GO

SELECT
	schemas.name AS SchemaName,
	tables.name AS TableName,
	columns.name AS ColumnName,
	types.name AS DataTypeName,
	columns.max_length,
	columns.precision,
	columns.scale,
	columns.is_nullable
FROM sys.tables
INNER JOIN sys.columns
ON tables.object_id = columns.object_id
INNER JOIN sys.types
ON types.user_type_id = columns.user_type_id
INNER JOIN sys.schemas
ON schemas.schema_id = tables.schema_id
WHERE tables.is_ms_shipped = 0;

EXEC sp_addextendedproperty
	@name = 'Employee Pay History Rate',
	@value = 'An employees pay rate.  This will contain up to 4 decimal places and can never be NULL.',
	@level0type = 'Schema', @level0name = 'HumanResources',
	@level1type = 'Table', @level1name = 'EmployeePayHistory',
	@level2type = 'Column', @level2name = 'Rate';

SELECT
	*
FROM sys.extended_properties
INNER JOIN sys.objects
ON extended_properties.major_id = objects.object_id
INNER JOIN sys.columns
ON columns.object_id = objects.object_id
AND columns.column_id = extended_properties.minor_id
WHERE objects.name = 'EmployeePayHistory'
AND columns.name = 'Rate'
GO

CREATE TABLE dbo.data_element_detail
(	data_element_id INT IDENTITY(1,1) NOT NULL CONSTRAINT PK_data_element_detail PRIMARY KEY CLUSTERED,
	data_element_name VARCHAR(100) NOT NULL,
	data_element_type VARCHAR(25) NOT NULL,
	data_type VARCHAR(128) NOT NULL,
	data_purpose VARCHAR(100) NOT NULL,
	entity_purpose VARCHAR(300) NOT NULL,
	database_server VARCHAR(128) NOT NULL,
	database_name VARCHAR(128) NOT NULL,
	schema_name VARCHAR(128) NOT NULL,
	table_name VARCHAR(128) NOT NULL,
	column_name VARCHAR(128) NOT NULL,
	original_data_source VARCHAR(500) NOT NULL,
	notes VARCHAR(1000) NOT NULL,
	foreign_key_to VARCHAR(128) NULL,
	row_count BIGINT NOT NULL,
	data_start_time SMALLDATETIME NULL,
	data_end_time SMALLDATETIME NULL,
	create_time DATETIME2(3) NULL,
	last_update_time DATETIME2(3) NOT NULL,
	is_deleted BIT NOT NULL)

ALTER TABLE dbo.data_element_detail WITH CHECK ADD CONSTRAINT CK_data_element_detail_data_element_type CHECK ((data_element_type='Dimension' OR data_element_type='Fact'))
GO

ALTER TABLE dbo.data_element_detail WITH CHECK ADD CONSTRAINT CK_data_element_detail_start_end_time CHECK ((data_start_time IS NULL AND data_end_time IS NULL OR data_start_time IS NOT NULL AND data_end_time IS NOT NULL OR data_start_time IS NOT NULL AND data_end_time IS NULL))
GO

SELECT * FROM sys.foreign_keys
SELECT * FROM sys.foreign_key_columns

INSERT INTO dbo.data_element_detail
	(data_element_name, data_element_type, data_type, data_purpose, entity_purpose, database_server, database_name, schema_name, table_name,
     column_name, original_data_source, notes, foreign_key_to, row_count, data_start_time, data_end_time, create_time, last_update_time,
     is_deleted)
VALUES
(   'Employee Login ID',
    'Dimension',
    'NVARCHAR(256)',
    'AdventureWorks application data',
    'Details and login information for all employees.',
    'DevServer1',
    'AdventureWorks2017',
    'HumanResources',
    'Employee',
    'LoginID',
    '',
    'This is the full login ID for a given employee, including domain + ''\'' as a prefix to the username itself',
    NULL,
    290,
    '2021-02-02 19:46:54',
    '2021-02-02 19:46:54',
    SYSDATETIME(),
    SYSDATETIME(),
    NULL);

SELECT * FROM dbo.data_element_detail;

SELECT COUNT(*) FROM HumanResources.Employee

SELECT * FROM HumanResources.Employee
GO


UPDATE data_element_detail
	set notes = 'The unique ID used to identify a product.'
WHERE schema_name = 'Production'
and table_name = 'Product'
and column_name = 'ProductID';

SELECT
	SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, 1, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) - 1),
	SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1) - CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) - 1),
	SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1) + 1, LEN(REFERENCING_ELEMENT.foreign_key_to) - CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1)),
	REFERENCING_ELEMENT.notes
FROM dbo.data_element_detail REFERENCING_ELEMENT
INNER JOIN dbo.data_element_detail REFERENCED_ELEMENT
ON REFERENCING_ELEMENT.database_server = REFERENCED_ELEMENT.database_server
AND REFERENCING_ELEMENT.database_name = REFERENCED_ELEMENT.database_name
AND SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, 1, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) - 1) = REFERENCED_ELEMENT.schema_name
AND SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1) - CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) - 1) = REFERENCED_ELEMENT.table_name
AND SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1) + 1, LEN(REFERENCING_ELEMENT.foreign_key_to) - CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1)) = REFERENCED_ELEMENT.column_name
WHERE (REFERENCING_ELEMENT.notes IS NULL OR REFERENCING_ELEMENT.notes = '')
AND REFERENCING_ELEMENT.foreign_key_to IS NOT NULL
AND REFERENCED_ELEMENT.foreign_key_to IS NULL
AND REFERENCED_ELEMENT.notes IS NOT NULL;

UPDATE REFERENCING_ELEMENT
	SET notes = REFERENCED_ELEMENT.notes
FROM dbo.data_element_detail REFERENCING_ELEMENT
INNER JOIN dbo.data_element_detail REFERENCED_ELEMENT
ON REFERENCING_ELEMENT.database_server = REFERENCED_ELEMENT.database_server
AND REFERENCING_ELEMENT.database_name = REFERENCED_ELEMENT.database_name
AND SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, 1, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) - 1) = REFERENCED_ELEMENT.schema_name
AND SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1) - CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) - 1) = REFERENCED_ELEMENT.table_name
AND SUBSTRING(REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1) + 1, LEN(REFERENCING_ELEMENT.foreign_key_to) - CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to, CHARINDEX('.', REFERENCING_ELEMENT.foreign_key_to) + 1)) = REFERENCED_ELEMENT.column_name
WHERE REFERENCING_ELEMENT.notes = ''
AND REFERENCING_ELEMENT.foreign_key_to IS NOT NULL
AND REFERENCED_ELEMENT.foreign_key_to IS NULL
AND REFERENCED_ELEMENT.notes <> '';

select * from data_element_detail WHERE schema_name = 'Production'
and table_name = 'Product'
and column_name = 'ProductID';

SELECT
	*
FROM dbo.data_element_detail
WHERE data_element_detail.column_name = 'ProductID'
AND data_element_detail.table_name <> 'Product'
AND data_element_detail.notes <> '';

select CHARINDEX('.', 'Sales.Product.ProductID');
select CHARINDEX('.', 'Sales.Product.ProductID', CHARINDEX('.', 'Sales.Product.ProductID') + 1);

SELECT SUBSTRING('Sales.Product.ProductID', 1, CHARINDEX('.', 'Sales.Product.ProductID') - 1);
SELECT SUBSTRING('Sales.Product.ProductID', CHARINDEX('.', 'Sales.Product.ProductID') + 1, CHARINDEX('.', 'Sales.Product.ProductID', CHARINDEX('.', 'Sales.Product.ProductID') + 1) - CHARINDEX('.', 'Sales.Product.ProductID') - 1);
SELECT SUBSTRING('Sales.Product.ProductID', CHARINDEX('.', 'Sales.Product.ProductID', CHARINDEX('.', 'Sales.Product.ProductID') + 1) + 1, LEN('Sales.Product.ProductID') - CHARINDEX('.', 'Sales.Product.ProductID', CHARINDEX('.', 'Sales.Product.ProductID') + 1));

select SUBSTRING('Sales.Product.ProductID', CHARINDEX('.', 'Sales.Product.ProductID') + 1, 7)

CHARINDEX('.', 'Sales.Product.ProductID', 7)
CHARINDEX('.', 'Sales.Product.ProductID')
)
select 'Sales.Product.ProductID'
select 'Sales.Product.ProductID'
