ALTER TABLE data_element_detail ALTER COLUMN foreign_key_to VARCHAR(384) NULL;
GO

CREATE OR ALTER PROCEDURE dbo.update_data_dictionary
	@database_list dbo.database_list READONLY
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @sql_command NVARCHAR(MAX);

	CREATE TABLE #database_list
	(	database_name VARCHAR(128) NOT NULL,
		is_processed BIT NOT NULL);

	INSERT INTO #database_list
		(database_name, is_processed)
	SELECT
		databases.name AS database_name,
		0 AS is_processed
	FROM sys.databases
	INNER JOIN @database_list database_list
	ON databases.name = database_list.database_name;

	CREATE TABLE #row_count
	(	database_server VARCHAR(128) NOT NULL,
		database_name VARCHAR(128) NOT NULL,
		schema_name VARCHAR(128) NOT NULL,
		table_name VARCHAR(128) NOT NULL,
		row_count BIGINT NOT NULL);

	CREATE TABLE #foreign_key_metadata
	(	database_server VARCHAR(128) NOT NULL,
		database_name VARCHAR(128) NOT NULL,
		schema_name VARCHAR(128) NOT NULL,
		table_name VARCHAR(128) NOT NULL,
		column_name VARCHAR(128) NOT NULL,
		foreign_key_column_number INT NOT NULL,
		foreign_key_to VARCHAR(384) NOT NULL,
		foreign_key_column_count INT NOT NULL);

	CREATE TABLE #data_element_list_to_process
	(	data_type VARCHAR(128) NOT NULL, -- INT, DECIMAL, VARCHAR(25), etc...
		database_server VARCHAR(128) NOT NULL,
		database_name VARCHAR(128) NOT NULL,
		schema_name VARCHAR(128) NOT NULL,
		table_name VARCHAR(128) NOT NULL,
		column_name VARCHAR(128) NOT NULL,
		row_count BIGINT NOT NULL,
		is_nullable BIT NOT NULL,
		foreign_key_to VARCHAR(384) NULL);
	
	DECLARE @current_database_name VARCHAR(MAX);

	WHILE EXISTS (SELECT * FROM #database_list WHERE is_processed = 0)
	BEGIN
		SELECT TOP 1
			@current_database_name = database_list.database_name
		FROM #database_list database_list
		WHERE database_list.is_processed = 0;

		SELECT @sql_command = '
			INSERT INTO #row_count
				(database_server, database_name, schema_name, table_name, row_count)
			SELECT
				@@SERVERNAME AS database_server,
				''' + @current_database_name + ''' AS database_name,
				schemas.name AS schema_name,
				tables.name AS table_name,
				SUM(dm_db_partition_stats.row_count) AS row_count
			FROM [' + @current_database_name + '].sys.dm_db_partition_stats
			INNER JOIN [' + @current_database_name + '].sys.tables
			ON tables.object_id = dm_db_partition_stats.object_id
			INNER JOIN [' + @current_database_name + '].sys.indexes
			ON indexes.object_id = dm_db_partition_stats.object_id
			AND indexes.index_id = dm_db_partition_stats.index_id
			INNER JOIN [' + @current_database_name + '].sys.schemas
			ON schemas.schema_id = tables.schema_id
			WHERE indexes.index_id IN (0, 1) -- 0 (Heap), 1 (Clustered Index)
			GROUP BY schemas.name, tables.name;

			INSERT INTO #foreign_key_metadata
				(database_server, database_name, schema_name, table_name, column_name, foreign_key_column_number,
				 foreign_key_to, foreign_key_column_count)
			SELECT
				@@SERVERNAME AS database_server,
				''' + @current_database_name + ''' AS database_name,
				REFERENCING_SCHEMA.name AS schema_name,
				REFERENCING_TABLE.name AS table_name,
				REFERENCING_COLUMN.name AS column_name,
				foreign_key_columns.constraint_column_id AS foreign_key_column_number,
				REFERENCED_SCHEMA.name + ''.'' + REFERENCED_TABLE.name + ''.'' + REFERENCED_COLUMN.name AS foreign_key_to,
				(SELECT COUNT(*) FROM [' + @current_database_name + '].sys.foreign_key_columns COLUMN_CHECK WHERE COLUMN_CHECK.constraint_object_id = FOREIGN_KEY.object_id) AS column_count
			FROM [' + @current_database_name + '].sys.foreign_key_columns
			INNER JOIN [' + @current_database_name + '].sys.objects FOREIGN_KEY
			ON FOREIGN_KEY.object_id = foreign_key_columns.constraint_object_id
			INNER JOIN [' + @current_database_name + '].sys.tables REFERENCING_TABLE
			ON foreign_key_columns.parent_object_id = REFERENCING_TABLE.object_id
			INNER JOIN [' + @current_database_name + '].sys.schemas REFERENCING_SCHEMA
			ON REFERENCING_SCHEMA.schema_id = REFERENCING_TABLE.schema_id
			INNER JOIN [' + @current_database_name + '].sys.columns REFERENCING_COLUMN
			ON foreign_key_columns.parent_object_id = REFERENCING_COLUMN.object_id 
			AND foreign_key_columns.parent_column_id = REFERENCING_COLUMN.column_id
			INNER JOIN [' + @current_database_name + '].sys.columns REFERENCED_COLUMN
			ON foreign_key_columns.referenced_object_id = REFERENCED_COLUMN.object_id
			AND foreign_key_columns.referenced_column_id = REFERENCED_COLUMN.column_id
			INNER JOIN [' + @current_database_name + '].sys.tables REFERENCED_TABLE
			ON REFERENCED_TABLE.object_id = foreign_key_columns.referenced_object_id
			INNER JOIN [' + @current_database_name + '].sys.schemas REFERENCED_SCHEMA
			ON REFERENCED_SCHEMA.schema_id = REFERENCED_TABLE.schema_id;
			
			INSERT INTO #data_element_list_to_process
				(data_type, database_server, database_name, schema_name, table_name, column_name, row_count, is_nullable, foreign_key_to)
			SELECT
				CASE
					WHEN types.name IN (''VARCHAR'', ''CHAR'') THEN types.name + ''('' + CASE WHEN columns.max_length = -1 THEN ''MAX'' ELSE CAST(columns.max_length AS VARCHAR(MAX)) END + '')''
					WHEN types.name IN (''NVARCHAR'', ''NCHAR'') THEN types.name + ''('' + CASE WHEN columns.max_length = -1 THEN ''MAX'' ELSE CAST(columns.max_length / 2 AS VARCHAR(MAX)) END + '')''
					WHEN types.name IN (''DECIMAL'', ''NUMERIC'') THEN types.name + ''('' + CAST(columns.precision AS VARCHAR(MAX)) + '','' + CAST(columns.scale AS VARCHAR(MAX)) + '')''
					WHEN types.name = ''DATETIME2'' THEN types.name + ''('' + CAST(columns.scale AS VARCHAR(MAX)) + '')''
					ELSE types.name
				END AS data_type,
				@@SERVERNAME AS database_server,
				''' + @current_database_name + ''' AS database_name,
				schemas.name AS schema_name,
				tables.name AS table_name,
				columns.name AS column_name,
				row_count.row_count,
				columns.is_nullable,
				foreign_key_metadata.foreign_key_to + CASE
					WHEN foreign_key_metadata.foreign_key_column_count > 1
						THEN '' ('' + CAST(foreign_key_metadata.foreign_key_column_number AS VARCHAR(MAX)) + '' of '' +
							 CAST(foreign_key_metadata.foreign_key_column_count AS VARCHAR(MAX)) + '')''
						ELSE ''''
					END AS foreign_key_to
			FROM [' + @current_database_name + '].sys.tables
			INNER JOIN [' + @current_database_name + '].sys.schemas
			ON schemas.schema_id = tables.schema_id
			INNER JOIN [' + @current_database_name + '].sys.columns
			ON tables.object_id = columns.object_id
			INNER JOIN [' + @current_database_name + '].sys.types
			ON columns.user_type_id = types.user_type_id
			INNER JOIN #row_count row_count
			ON row_count.table_name = tables.name COLLATE DATABASE_DEFAULT
			AND row_count.schema_name = schemas.name COLLATE DATABASE_DEFAULT
			AND row_count.database_name = ''' + @current_database_name + '''
			AND row_count.database_server = @@SERVERNAME
			LEFT JOIN #foreign_key_metadata foreign_key_metadata
			ON foreign_key_metadata.column_name = columns.name COLLATE DATABASE_DEFAULT
			AND foreign_key_metadata.table_name = tables.name COLLATE DATABASE_DEFAULT
			AND foreign_key_metadata.schema_name = schemas.name COLLATE DATABASE_DEFAULT
			AND foreign_key_metadata.database_name = ''' + @current_database_name + '''
			AND foreign_key_metadata.database_server = @@SERVERNAME;';

		EXEC sp_executesql @sql_command;

		UPDATE database_list
			SET is_processed = 1
		FROM #database_list database_list
		WHERE database_list.database_name = @current_database_name;
	END

	MERGE INTO dbo.data_element_detail AS TARGET
	USING (SELECT
				data_type,
				database_server,
				database_name,
				schema_name,
				table_name,
				column_name,
				row_count,
				is_nullable,
				dbo.format_entity_name(column_name) AS data_element_name,
				CASE WHEN column_name LIKE 'dim%' THEN 'Dimension'
					 WHEN column_name LIKE 'fact%' THEN 'Fact'
					 ELSE ''
				END AS data_element_type,
				'' AS data_purpose,
				'' AS entity_purpose,
				'' AS original_data_source,
				'' AS notes,
				data_element_list_to_process.foreign_key_to AS foreign_key_to,
				NULL AS data_start_time,
				NULL AS data_end_time
			FROM #data_element_list_to_process data_element_list_to_process) AS SOURCE
		ON (SOURCE.table_name = TARGET.table_name
		AND SOURCE.column_name = TARGET.column_name
		AND SOURCE.schema_name = TARGET.schema_name
		AND SOURCE.database_name = TARGET.database_name
		AND SOURCE.database_server = TARGET.database_server)
	WHEN NOT MATCHED BY TARGET
		THEN INSERT
			(data_element_name, data_element_type, data_type, data_purpose, entity_purpose, database_server, database_name, schema_name,
			  table_name, column_name, original_data_source, notes, foreign_key_to, row_count, data_start_time,
			  data_end_time, create_time, last_update_time, is_deleted, is_nullable)
		VALUES
		(	SOURCE.data_element_name,
			SOURCE.data_element_type,
			SOURCE.data_type,
			SOURCE.data_purpose,
			SOURCE.entity_purpose,
			SOURCE.database_server,
			SOURCE.database_name,
			SOURCE.schema_name,
			SOURCE.table_name,
			SOURCE.column_name,
			SOURCE.original_data_source,
			SOURCE.notes,
			SOURCE.foreign_key_to,
			SOURCE.row_count,
			SOURCE.data_start_time,
			SOURCE.data_end_time,
			GETUTCDATE(), -- create_time
			GETUTCDATE(), -- last_update_time
			0, -- is_deleted
			SOURCE.is_nullable)
	WHEN MATCHED AND (SOURCE.data_type <> TARGET.data_type OR SOURCE.row_count <> TARGET.row_count OR SOURCE.is_nullable <> TARGET.is_nullable
					  OR ISNULL(SOURCE.foreign_key_to, '') <> ISNULL(TARGET.foreign_key_to, '') OR TARGET.is_deleted = 1)
		THEN UPDATE
			SET data_type = SOURCE.data_type,
				row_count = SOURCE.row_count,
				is_nullable = SOURCE.is_nullable,
				foreign_key_to = SOURCE.foreign_key_to,
				is_deleted = 0,
				last_update_time = GETUTCDATE()
	WHEN NOT MATCHED BY SOURCE
		THEN UPDATE
			SET is_deleted = 1,
				last_update_time = GETUTCDATE();

	DROP TABLE #database_list;
	DROP TABLE #row_count;
	DROP TABLE #data_element_list_to_process;
	DROP TABLE #foreign_key_metadata;
END
GO

DECLARE @database_list dbo.database_list;
INSERT INTO @database_list
	(database_name)
VALUES
	('AdventureWorks2017'),
	('WideWorldImporters'),
	('WideWorldImportersDW');

EXEC dbo.update_data_dictionary @database_list = @database_list;

SELECT
	*
FROM dbo.data_element_detail
where foreign_key_to IS NOT NULL

